package xyz.lychee.lagfixer.libs.adventure.text.minimessage.internal.parser.node;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.internal.parser.Token;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.internal.parser.TokenType;
import xyz.lychee.lagfixer.libs.adventure.text.minimessage.tree.Node;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

public class ElementNode implements Node {
   private final @Nullable ElementNode parent;
   private final @Nullable Token token;
   private final String sourceMessage;
   private final List<ElementNode> children = new ArrayList();

   ElementNode(final @Nullable ElementNode parent, final @Nullable Token token, final @NotNull String sourceMessage) {
      this.parent = parent;
      this.token = token;
      this.sourceMessage = sourceMessage;
   }

   public @Nullable ElementNode parent() {
      return this.parent;
   }

   public @Nullable Token token() {
      return this.token;
   }

   public @NotNull String sourceMessage() {
      return this.sourceMessage;
   }

   public @NotNull List<ElementNode> children() {
      return Collections.unmodifiableList(this.children);
   }

   public @NotNull List<ElementNode> unsafeChildren() {
      return this.children;
   }

   public void addChild(final @NotNull ElementNode childNode) {
      int last = this.children.size() - 1;
      if (childNode instanceof TextNode && !this.children.isEmpty() && this.children.get(last) instanceof TextNode) {
         TextNode lastNode = (TextNode)this.children.remove(last);
         if (lastNode.token().endIndex() == childNode.token().startIndex()) {
            Token replace = new Token(lastNode.token().startIndex(), childNode.token().endIndex(), TokenType.TEXT);
            this.children.add(new TextNode(this, replace, lastNode.sourceMessage()));
         } else {
            this.children.add(lastNode);
            this.children.add(childNode);
         }
      } else {
         this.children.add(childNode);
      }

   }

   public @NotNull StringBuilder buildToString(final @NotNull StringBuilder sb, final int indent) {
      char[] in = this.ident(indent);
      sb.append(in).append("Node {\n");
      Iterator var4 = this.children.iterator();

      while(var4.hasNext()) {
         ElementNode child = (ElementNode)var4.next();
         child.buildToString(sb, indent + 1);
      }

      sb.append(in).append("}\n");
      return sb;
   }

   char @NotNull [] ident(final int indent) {
      char[] c = new char[indent * 2];
      Arrays.fill(c, ' ');
      return c;
   }

   public @NotNull String toString() {
      return this.buildToString(new StringBuilder(), 0).toString();
   }
}
